package space.eptaproject.vmodaddon;

import it.unimi.dsi.fastutil.ints.IntArrayList;
import it.unimi.dsi.fastutil.ints.IntList;
import it.unimi.dsi.fastutil.objects.ObjectArrayList;
import it.unimi.dsi.fastutil.objects.ObjectImmutableList;
import net.minecraft.resources.ResourceLocation;
import net.minecraft.world.item.ItemStack;
import net.minecraftforge.common.ForgeConfigSpec;
import net.minecraftforge.registries.ForgeRegistries;

import java.util.*;

public class Config {
    public static final ForgeConfigSpec COMMON_CONFIG;
    public static final ForgeConfigSpec.ConfigValue<List<? extends String>> RESOURCE_LOCATION_LIST;
    public static final ForgeConfigSpec.ConfigValue<List<? extends String>> ITEM_BLACK_LIST;
    public static final ForgeConfigSpec.ConfigValue<String> RESOURCE_LOCATION;

    public static List<ResourceLocation> parsedList = new ObjectArrayList<>();
    private static List<Map.Entry<Integer, ResourceLocation>> ITEM = new ObjectImmutableList<>(new ArrayList<>());
    public static ResourceLocation replace = Validator.RB;

    static {
        ForgeConfigSpec.Builder builder = new ForgeConfigSpec.Builder();

        builder.push("Schematic");

        RESOURCE_LOCATION_LIST = builder
                .comment("List of blocked blocks, format: 'modid:path'")
                .defineListAllowEmpty(
                        "blocked",
                        List.of("minecraft:stone", "minecraft:diamond_block"),
                        obj -> obj instanceof String && ResourceLocation.isValidResourceLocation((String) obj)
                );

        RESOURCE_LOCATION = builder
                .comment("replace to, format: 'modid:path'")
                .define(
                        "replace",
                        Validator.RB.toString()
                );

        builder.pop();

        builder.push("Inventory");

        ITEM_BLACK_LIST = builder
                .comment("List of blocked items, format: 'modid:path'")
                .defineListAllowEmpty(
                        "blocked",
                        List.of("minecraft:stone", "minecraft:diamond_block"),
                        obj -> obj instanceof String && ResourceLocation.isValidResourceLocation((String) obj)
                );

        builder.pop();
        COMMON_CONFIG = builder.build();
    }

    public static void bake() {
        parsedList = RESOURCE_LOCATION_LIST.get().stream()
                .map(s -> {
                    try {
                        return ResourceLocation.parse(s);
                    } catch (Exception e) {
                        return null;
                    }
                })
                .filter(Objects::nonNull)
                .toList();

        ITEM = new ObjectImmutableList(ITEM_BLACK_LIST.get().stream()
                .map(s -> {
                    try {
                        ResourceLocation location = ResourceLocation.parse(s);
                        return new AbstractMap.SimpleEntry<Integer, ResourceLocation>(location.hashCode(), location);
                    } catch (Exception e) {
                        return null;
                    }
                })
                .filter(Objects::nonNull)
                .toList());
        try {
            replace = ResourceLocation.parse(RESOURCE_LOCATION.get());
        } catch (Exception e) {
            replace = Validator.RB;
        }

    }

    public static boolean contains(ItemStack stack) {
        if (stack.isEmpty()) return false;
        ResourceLocation id = ForgeRegistries.ITEMS.getKey(stack.getItem());
        if (id == null) return false;

        int hash = id.hashCode();
        for (var entry : ITEM) {
            if (entry.getKey() == hash && entry.getValue().equals(id)) {
                return true;
            }
        }
        return false;
    }
}
